@@ -13,6 +13,7 @@
 #include "update.h"
 #include "export.h"
 #include "log.h"
+#include "cat.h"
 #include "ignore.h"
 #include "cp_mv.h"
 #include "sync.h"
@@ -35,32 +36,35 @@
 /** Array of command name pointers.
  * The \c acl at the beginning means <i>AC</i>tion <i>L</i>ist. */
 static const char 
-			*acl_status[]     = { "status", NULL },
-			*acl_commit[]     = { "commit", "checkin", "ci", NULL },
-			*acl_update[]     = { "update", NULL },
-			*acl_export[]     = { "export", NULL },
-			*acl_build[]      = { "_build-new-list", NULL },
-			*acl_remote[]     = { "remote-status", "rs", NULL },
-			*acl_ignore[]     = { "ignore", NULL },
-			*acl_add[]        = { "add", NULL },
-			*acl_copyfr[]     = { "copyfrom-detect", "copy-detect", NULL },
-			*acl_cp[]         = { "copy", "move", "cp", "mv", NULL },
-			*acl_unvers[]     = { "unversion", NULL },
-			*acl_log[]        = { "log", NULL },
-			*acl_resolv[]     = { "resolved", NULL },
-			*acl_checko[]   	= { "checkout", "co", NULL },
-			*acl_sync_r[]     = { "sync-repos", NULL },
-			*acl_revert[]     = { "revert", "undo", NULL },
-			*acl_prop_l[]     = { "prop-list", "pl", NULL },
-			*acl_prop_g[]     = { "prop-get", "pg", NULL },
-			*acl_prop_s[]     = { "prop-set", "ps", NULL },
-			*acl_prop_d[]     = { "prop-del", "pd", NULL },
-			*acl_diff[]       = { "diff", NULL },
-			*acl_help[]       = { "help", "?", NULL },
-			*acl_mergelist[] UNUSED = { "mergelist", NULL },
-			*acl_info[]       = { "info", NULL },
+			*acl_status[] = { "status", NULL },
+			*acl_commit[] = { "commit", "checkin", "ci", NULL },
+			*acl_update[] = { "update", NULL },
+			*acl_export[] = { "export", NULL },
+			*acl_build[]  = { "_build-new-list", NULL },
+			*acl_delay[]   = { "delay", NULL },
+			*acl_remote[] = { "remote-status", "rs", NULL },
+			*acl_ignore[] = { "ignore", NULL },
+			*acl_rign[]   = { "rel-ignore", "ri", "r-i", NULL },
+			*acl_add[]    = { "add", NULL },
+			*acl_copyfr[] = { "copyfrom-detect", "copy-detect", NULL },
+			*acl_cp[]     = { "copy", "move", "cp", "mv", NULL },
+			*acl_uncp[]   = { "uncopy", NULL },
+			*acl_unvers[] = { "unversion", NULL },
+			*acl_log[]    = { "log", NULL },
+			*acl_cat[]    = { "cat", NULL },
+			*acl_resolv[] = { "resolved", NULL },
+			*acl_checko[] = { "checkout", "co", NULL },
+			*acl_sync_r[] = { "sync-repos", NULL },
+			*acl_revert[] = { "revert", "undo", NULL },
+			*acl_prop_l[] = { "prop-list", "pl", NULL },
+			*acl_prop_g[] = { "prop-get", "pg", NULL },
+			*acl_prop_s[] = { "prop-set", "ps", NULL },
+			*acl_prop_d[] = { "prop-del", "pd", NULL },
+			*acl_diff[]   = { "diff", NULL },
+			*acl_help[]   = { "help", "?", NULL },
+			*acl_info[]   = { "info", NULL },
 			/** \todo: remove initialize */
-			*acl_urls[]       = { "urls", "initialize", NULL };
+			*acl_urls[]   = { "urls", "initialize", NULL };
 
 
 /* A generated file. */
@@ -75,8 +79,11 @@ static const char
 	.work=_work, .local_callback=_act, \
 	__VA_ARGS__ }
 
+/** Use the progress uninitializer */
 #define UNINIT .local_uninit=st__progress_uninit
+/** Store update-pipe strings */
 #define DECODER .needs_decoder=1
+/** Commands obeys filtering via -f */
 #define FILTER .only_opt_filter=1
 
 
@@ -93,14 +100,18 @@ struct actionlist_t action_list[]=
 	ACT(  diff,   df__work,         NULL, DECODER),
 	ACT(sync_r, sync__work,         NULL, .repos_feedback=sync__progress, .keep_user_prop=1),
 	ACT(  urls,  url__work,         NULL),
-	ACT(revert,  rev__work,  rev__action, UNINIT, DECODER, .keep_children=1),
+	ACT(revert,  rev__work,         NULL, UNINIT, DECODER, .keep_children=1),
 	ACT(ignore,  ign__work,         NULL),
+	ACT(  rign,  ign__rign,         NULL),
 	ACT(copyfr, cm__detect, st__progress, UNINIT),
 	ACT(    cp,   cm__work,         NULL),
+	ACT(   cat,  cat__work,         NULL),
+	ACT(  uncp, cm__uncopy,         NULL),
 	ACT(resolv,  res__work,  res__action, .is_compare=1),
 	ACT(   log,  log__work,         NULL),
 	ACT(checko,   co__work,         NULL, DECODER, .repos_feedback=st__rm_status),
 	ACT( build,  bld__work,   st__status),
+	ACT( delay,delay__work,   st__status),
 	/* For help we set import_export, to avoid needing a WAA 
 	 * (default /var/spool/fsvs) to exist. */
 	ACT(  help,  ac__Usage,         NULL, .is_import_export=1),
