@@ -137,6 +137,13 @@ struct ignore_t {
 		};
 	};
 
+	/** AND-value for mode matching, or \c 0 for not chosen. */
+	unsigned short mode_match_and;
+	/** CMP-value for mode matching. */
+	unsigned short mode_match_cmp;
+
+	/** Should this match only directories? */
+	unsigned int dir_only:1;
 	/** Is this an ignore or take pattern?  \a 0 = take, \a 1 = ignore */
 	unsigned int is_ignore:1;
 	/** Ignore case for comparing? */
@@ -273,6 +280,9 @@ struct sstat_t {
  *
  * The name comes from <em>extended struct stat</em>.
  * This structure is used to build the tree of entries that we're processing.
+ *
+ * We need both a local and a remote status, to see on update when there 
+ * might be a conflict. \todo Single status, and check entries on-time?
  */
 struct estat {
 	/** The parent of this entry, used for tree walking.
@@ -303,13 +313,12 @@ struct estat {
 	union {
 		/** For files */
 		struct {
-			/** MD5-hash of the repository version. 
-			 * While committing it is set to the \e new MD5, and saved 
-			 * with \a waa__output_tree(). */
-			md5_digest_t md5;
 			/** The decoder string from fsvs:update-pipe. Only gets set if 
 			 * action->needs_decoder != 0. */
 			char *decoder;
+			/** MD5-hash of the repository version.  While committing it is set 
+			 * to the \e new MD5, and saved with \a waa__output_tree(). */
+			md5_digest_t md5;
 			/** Whether we got an "original" MD5 from the repository to compare.  
 			 * */
 			unsigned int has_orig_md5:1;
@@ -318,8 +327,16 @@ struct estat {
 			 * See \ref ChgFlag. */
 			unsigned int change_flag:2;
 		};
-		/** For directories */
+		/** For directories.
+		 * The by_inode and by_name members are positioned so that they collide 
+		 * with the \c md5 file member above - in case of incorrect file types 
+		 * that's safer, as they'll contain invalid pointers instead of (the 
+		 * valid) \c decoder.  */
 		struct {
+			/** Name storage space for sub- (and sub-sub- etc.) entries.
+			 * Mainly used in the root inode, but is used in newly found directories
+			 * too. \c NULL for most directory entries. */
+			char *strings;
 			/** List of child entries.
 			 * Sorted by inode number, NULL-terminated.
 			 * Always valid. */
@@ -329,13 +346,16 @@ struct estat {
 			 * May be NULL if not currently used; can be (re-)generated by calling
 			 * dir__sortbyname(). */
 			struct estat **by_name;
-			/** Name storage space for sub- (and sub-sub- etc.) entries.
-			 * Mainly used in the root inode, but is used in newly found directories
-			 * too. \c NULL for most directory entries. */
-			char *strings;
 
 			/** How many entries this directory has. */
-			unsigned entry_count;
+			AC_CV_C_UINT32_T entry_count;
+
+			/** Used to know when this directories' children are finished.
+			 * Counts the number of unfinished subdirectories.
+			 * This is volatile and should be in the union below (with \ref 
+			 * estat::child_index), but as it's only used for directories it 
+			 * conserves memory to keep it here. */
+			AC_CV_C_UINT32_T unfinished;
 
 			/** This flag is set if any child is *not* at the same revision,
 			 * so this directory has to be descended on reporting. */
@@ -373,11 +393,11 @@ struct estat {
 		};
 		struct {
 			/** Used in waa__input_tree() and waa__update_tree(). */
-			unsigned child_index;
+			AC_CV_C_UINT32_T child_index;
 		};
 		/* output_tree */
 		struct {
-			unsigned file_index;
+			AC_CV_C_UINT32_T file_index;
 		};
 	};
 
@@ -393,11 +413,18 @@ struct estat {
 	 * The subpool is available from a hash lookup with key "" (len=0). */
   apr_hash_t *user_prop;
 
+	/** Updated unix mode from ops__update_single_entry().
+	 * See the special \ref fsvsS_constants below.
+	 * \todo Strip that to only the needed bits, ie. the ones (used) in \c 
+	 * S_IFMT, and make estat compact, by putting between the bitfields.
+	 * Convention: has \b always to be set according to the current type of 
+	 * the entry (to account for the shared members, eg. by_inode); where 
+	 * estat::st.mode has the \e original value, as seen by the repository. */
+	mode_t updated_mode;
+
 	/** Flags for this entry. See \ref EntFlags for constant definitions. */
-	int flags;
+	AC_CV_C_UINT32_T flags;
 
-	/** What kind of entry this is - see \ref ent_types. */
-	unsigned int entry_type:6;
 	/** Local status of this entry - \ref fs_bits. */
 	unsigned int entry_status:10;
 
@@ -428,15 +455,39 @@ struct estat {
 
 	/** Flag saying whether this entry was specified by the user on the 
 	 * command line. */
-	unsigned int do_tree:1;
-	/** Like \a FS_CHILD_CHANGED - children of this entry must be handled.
-	 * \todo substitute by \a FS_CHILD_CHANGED? */
-	unsigned int do_a_child:1;
-	/** Flag derived from parents' \ref estat::do_tree.
+	unsigned int do_userselected:1;
+	/** Says that a child of this entry was given by the user on the 
+	 * commandline.
+	 * Unlike \a FS_CHILD_CHANGED, which is set if some child has \e actually 
+	 * changed, this just says that we have to check. */
+	unsigned int do_child_wanted:1;
+	/** Flag derived from parents' \ref estat::do_userselected.
 	 * Set for \b all entries which should be handled. */
 	unsigned int do_this_entry:1;
+	/** Flag saying whether the \c "-f" filter condition applies.
+	 * Normally set in \ref ops__set_todo_bits(), can be cleared in \ref 
+	 * ops__update_filter_set_bits(). */
+	unsigned int do_filter_allows:1;
+	/** Flag used for debugging. If estat::do_filter_allows is queried 
+	 * without being defined earlier, we trigger a \ref BUG().
+	 * Was conditionalized on \c ENABLE_DEBUG - but that got ugly. */
+	unsigned int do_filter_allows_done:1;
+
+	/** Whether this entry should not be written into the \ref dir "entry 
+	 * list", and/or ignored otherwise. */
+	unsigned int to_be_ignored:1;
 };
 
+/** \anchor fsvsS_constants Special FSVS file type constants.
+ * @{ */
+#define S_IFUNDEF (0)
+/** All sockets get filtered out when the directory gets read, so we can 
+ * safely reuse that value for the case where we don't know \b what kind of 
+ * special entry that is (eg when receiving \c "svn:special" from the 
+ * repository). */
+#define S_IFANYSPECIAL S_IFSOCK
+#define S_ISANYSPECIAL S_ISSOCK
+/** @} */
 
 /** \anchor EntFlags Various flags for entries.
  *
@@ -484,22 +535,6 @@ struct estat {
 #define RF___IS_COPY (RF_COPY_BASE | RF_COPY_SUB)
 
 
-/** \anchor ent_types Entry types.
- * Historically derived from filetypes.  */
-#define FT_UNKNOWN (0)
-#define FT_DIR (1)
-#define FT_FILE (2)
-#define FT_CDEV (4)
-#define FT_BDEV (8)
-#define FT_SYMLINK (16)
-// sockets ?
-#define FT_IGNORE (32) 
-#define FT_ANY (FT_DIR | FT_FILE | FT_CDEV | FT_BDEV | FT_SYMLINK | FT_IGNORE)
-#define FT_NONDIR (FT_ANY & ~FT_DIR)
-#define FT_ANYSPECIAL (FT_NONDIR & ~FT_FILE)
-
-#define FT__MASK (FT_ANY)
-
 
 /** \name File statii.
  * \anchor fs_bits
@@ -531,8 +566,8 @@ struct estat {
 		FS_META_GROUP | FS_META_UMODE)
 
 /** This flag on a directory entry says that the directory itself was
- * not changed, but some child, so this directories children have to be 
- * checked for modifications. */
+ * not changed, but some child, so the children of this directory
+ * have to be checked for modifications. */
 #define FS_CHILD_CHANGED (1 << 9)
 
 #define FS__CHANGE_MASK (FS_NEW | FS_REMOVED | FS_CHANGED | \
@@ -704,7 +739,7 @@ __attribute__ ((format (printf, 5, 6) ));
  * \endcode
  */ 
 /** The master error macro for calling subversion functions. */
-#define STOPIF_SVNERR_EXTRA(func, parm, fmt, ...)          \
+#define STOPIF_SVNERR_TEXT(func, parm, fmt, ...)          \
 	do                                                       \
 {                                                          \
 	status_svn=func parm;                                    \
@@ -718,10 +753,7 @@ __attribute__ ((format (printf, 5, 6) ));
  *   STOPIF_SVNERR( svn_ra_initialize, (global_pool));
  * \endcode
  */
-#define STOPIF_SVNERR(func, parm) STOPIF_SVNERR_EXTRA(func, parm, #func)
-/** The same as STOPIF_SVNERR(), but with a variable printed function name.
- * Used in case the function is called indirectly; see STOPIF_SVNERR_INDIR. */
-#define STOPIF_SVNERR_TEXT(func, text, parm) STOPIF_SVNERR_EXTRA(func, parm, text)
+#define STOPIF_SVNERR(func, parm) STOPIF_SVNERR_TEXT(func, parm, #func)
 /** Convert the svn_error_t into a message and a returnable integer. */
 #define STOP_HANDLE_SVNERR(svnerr) STOPIF_CODE_ERR_GOTO(svnerr, svnerr->apr_err, ex2, (const char*)svnerr->message)
 
@@ -739,7 +771,7 @@ __attribute__ ((format (printf, 5, 6) ));
 /** Makes the program abort.
  * If the configure had --enable-debug and \c gdb is in the path, try
  * to use \c gdb to debug this problem (only if STDIN and STDOUT are ttys). */
-#define BUG(...) do { debuglevel=1; DEBUGP(__VA_ARGS__); *(int*)42=__LINE__; } while (0)
+#define BUG(...) do { fflush(NULL); debuglevel=1; DEBUGP(__VA_ARGS__); *(int*)42=__LINE__; } while (0)
 /** The same as BUG(), but conditionalized. 
  * \code
  *   BUG_ON(a == b, "HELP")
@@ -781,13 +813,9 @@ __attribute__ ((format (printf, 5, 6) ));
  * A list of variables that can be set by commandline parameters or 
  * environment variables; these are used in nearly every action. */
 /** @{ */
-/** If the user wants to make sure whether files have been modified
- * (and the modification time check is not enough).
- * Makes checking slower, as data gets MD5ed. */
-extern int opt_checksum, 
-			 /** Greater than zero if additional details are wanted, 
-				* or negative for extra quiet operation. */
-			 opt_verbose, 
+/** Greater than zero if additional details are wanted, or negative for 
+ * extra quiet operation. */
+extern int opt_verbose, 
 			 /** Flag for recursive/non-recursive behaviour.
 				* Starting with 0, gets incremented with \c -R and decremented with \c 
 				* -N. Different actions have different default levels. */
